classdef ImageChangerMitch<mladapter
    % Same as ImageChanger monkeylogic adapter but the CurrentImageNumber
    % is now an accessible property. 
    properties
        ImageList
        DurationUnit='frame'
    end
    properties (Hidden)
        List
    end
    properties (SetAccess=protected)
        ImageID
        CurrentImageNum
    end
    properties (Access=protected)
        CountByFrame
        ImageSchedule
        
        PrevImageNum
    end
    methods
        function obj=ImageChangerMitch(varargin)
            obj=obj@mladapter(varargin{:});
        end
        function delete(obj)
            destroy_graphic(obj);
        end
        function val=get.List(obj)
            val=obj.ImageList;
        end
        function set.List(obj,val)
            obj.ImageList=val;
        end
        function set.ImageList(obj,val)
            ncol=size(val,2);
            if ncol<3||5<ncol
                error('ImageList must be an n-by-3 or n-by-4 or n-by-5 matrix');
            end
            obj.ImageList=cell(size(val,1),5);
            obj.ImageList(:,1:size(val,2))=val;
            create_graphic(obj);
        end
        function init(obj,p)
            init@mladapter(obj,p);
            obj.CountByFrame=strcmpi(obj.DurationUnit,'frame');
            obj.ImageSchedule=cumsum([obj.ImageList{:,3}]);
            obj.CurrentImageNum=0;
            obj.PrevImageNum=0;
        end
        function fini(obj,p)
            fini@mladapter(obj,p);
            deactivate_all(obj);
        end
        function continue_=analyze(obj,p)
            analyze@mladapter(obj,p);
            if obj.CountByFrame
                t=p.scene_frame();
            else
                t=p.scene_time();
            end
            obj.CurrentImageNum=find(t<obj.ImageSchedule,1);
            obj.Success=isempty(obj.CurrentImageNum);
            continue_=~obj.Success;
        end
        function draw(obj,p)
            draw@mladapter(obj,p);
            if isempty(obj.CurrentImageNum)
                deactivate_all(obj);
            elseif obj.PrevImageNum~=obj.CurrentImageNum
                if 0<obj.PrevImageNum
                    mglactivategraphic(obj.ImageID{obj.PrevImageNum},false);
                end
                obj.PrevImageNum=obj.CurrentImageNum;
                id=obj.ImageID{obj.CurrentImageNum};
                if ~isempty(id)
                    mglactivategraphic(id,true);
                end
                p.eventmarker(obj.ImageList{obj.CurrentImageNum,4});
            end
        end
    end
    methods (Access=protected)
        function create_graphic(obj)
            destroy_graphic(obj);
            nrow=size(obj.ImageList,1);
            obj.ImageID=cell(nrow,1);
            for m=1:nrow
                if isempty(obj.ImageList{m,1})
                    continue
                end
                switch class(obj.ImageList{m,1})
                    case 'double'
                        obj.ImageID{m}=obj.ImageList{m,1};
                        mglsetorigin(obj.ImageID{m},obj.Tracker.CalFun.deg2pix(obj.ImageList{m,2}));
                    case 'char'
                        imdata=mglimread(obj.ImageList{m,1});
                        if ~isempty(obj.ImageList{m,5})
                            imdata=mglimresize(imdata,obj.ImageList{m,5}([2,1]));
                        end
                        obj.ImageID{m}=mgladdbitmap(imdata);
                        mglsetorigin(obj.ImageID{m},obj.Tracker.CalFun.deg2pix(obj.ImageList{m,2}));
                    case 'cell'
                        nid=numel(obj.ImageList{m,1});
                        obj.ImageID{m}=NaN(1,nid);
                        nresize=size(obj.ImageList{m,5},1);
                        for n=1:nid
                            imdata=mglimread(obj.ImageList{m,1}{n});
                            if 0<nresize
                                if nresize<n
                                    row=nresize;
                                else
                                    row=n;
                                end
                                imdata=mglimresize(imdata,obj.ImageList{m,5}(row,[2,1]));
                            end
                            obj.ImageID{m}(n)=mgladdbitmap(imdata);
                        end
                        mglsetorigin(obj.ImageID{m},obj.Tracker.CalFun.deg2pix(obj.ImageList{m,2}));
                end
            end
            deactivate_all(obj);
        end
        function destroy_graphic(obj)
            if isempty(obj.ImageID)
                return
            end
            for m=1:size(obj.ImageList,1)
                switch class(obj.ImageList{m,1})
                    case {'char', 'cell'}
                        mgldestroygraphic(obj.ImageID{m});
                end
            end
            obj.ImageID=[];
        end
        function deactivate_all(obj)
            for m=1:size(obj.ImageList,1)
                mglactivategraphic(obj.ImageID{m},false);
            end
        end
    end
end
